
/* AVRGCC1.c
 *
 * -- ENGLISH --
 *
 * Note: This software is intended to be:
 *  - compiled with XCODE using AVRGCC on MAC OSX 10.5  
 *  - compiled with Atmel Studio on Windows 7
 *  - it can be easily adapted to other compilers on Linux system.
 *  - executed on Robotis (r) CM-510 Control Board.
 *    though it can be easily adapted to other Microcontroller Boards
 *
 * This code can be downloaded from : http://www.robotgarage.org/wiki/index.php?title=LionHell_McMillan
 * 
 * -------------------------------------------
 * Control Logic for LionHell McMillan Rough Terrain Wheg Robot
 *
 * 
 * Created by Vittorio Lumare on 10/07/2012.
 * Modified by Alessandro Rosina on 10/03/2015
 * --------------------------------------------
 *
 *
 Copyright 2012 Vittorio Lumare 
 Copyright 2015 Alessandro Rosina
 *
 *
 Contact: Vittorio Lumare (venom_at_venom_dot_it)
 Contact: Alessandro Rosina (rosina_at_alessandro_at_0_dot_gmail_at_com)
 *
 *
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */


#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <stdio.h>

#include "dynamixel.h"
#include "serial.h"

#include <math.h> //libm.a (doesn't work in winavr, works in avr gcc).

//
#define true 1
#define false 0

//Time Vars
#define TIME_SEC		15625
#define TIME_DECSEC		1563
#define TIME_CENTSEC	156
#define TIME_MILLISEC	16

// AX-12+ Control table addresses
#define P_GOAL_POSITION_L		30
#define P_GOAL_POSITION_H		31
#define P_PRESENT_POSITION_L	36
#define P_PRESENT_POSITION_H	37
#define P_PRESENT_SPEED_L		38
#define P_PRESENT_SPEED_H		39
#define P_PRESENT_LOAD_L		40
#define P_PRESENT_LOAD_H		41
#define P_MOVING				46
#define P_MOVING_SPEED_L		32
#define P_MOVING_SPEED_H		33
#define P_TORQUE_ENABLE			24

// AX S1 Control table addresses
#define LEFT_IR_SENSOR_DATA 26
#define CENT_IR_SENSOR_DATA 27
#define RIGH_IR_SENSOR_DATA 28
#define OBSTACLE_DETECTION_FLAG 32 
#define OBSTACLE_DETECTED_COMPARE_VALUE 20 //Obstacle Detection Threshold 
#define OBSTACLE_DETECTED_COMPARE 52 // 0: Low Dist Sens, 1: High Dist Sens
#define CENTER_LUMINOSITY 30 

//Button defines
#define BTN_UP 0x10
#define BTN_DOWN 0x20
#define BTN_LEFT 0x40
#define BTN_RIGHT 0x80
#define BTN_START 0x01

// Default setting
#define DEFAULT_BAUDNUM		1 // 1Mbps
#define DEFAULT_ID			1

struct ServoData{
	int id; 
	unsigned short lastP[7];//Master Servo: Present Position data
	_Bool db;//Dead Band state: 0 = not in db; 1 = in db;
	unsigned short spd; //Master Servo: Present Speed Data
};

//
// The structure of the parameters of the IR distance sensors
//
typedef const struct
{
	const signed short a;
	const signed short b;
	const signed short k;
}
ir_distance_sensor;


/* All sensor values */
typedef struct
	{
		float distance_front_center; //= readSharp_cm(4);
		float distance_floor_center; //= readSharp_cm(6);
		float distance_floor_left; //= readSharp_cm(5);
		float distance_floor_right; //= readSharp_cm(7);
		short body_inclination_xg; //= readAccDeg(1);
		short body_inclination_yg; //= readAccDeg(2);
		short body_inclination_zg; //= readAccDeg(0);
		short abdomen_inclination; //= getAbdomenDeg();
		short abdomen_inclination_g; //= getAbdomenDegG();
	}
	Sensors;



void init_OC1A(void);
void initTimer1(void);
unsigned int Timer1Read( void );
void Timer1Write( unsigned int i );
void Timer1Pause(unsigned int flag);/*Pause using timer 1 (warning: it resets timer!) */
void PrintCommStatus(int CommStatus);
void PrintErrorCode(void);
unsigned char readAXS1cd(void); 
void go_fwd(void);
void stop(void);
void torque(unsigned char val);
int checkDxlConn(void);
void adc_init(void); 
uint16_t adc_start(unsigned char channel);
float ir_distance_calculate_cm(ir_distance_sensor sensor, unsigned short adc_value);
unsigned short mirror(unsigned short speed);/* Mirror Speed: 0 to 1023, 1024 to 2047 */
unsigned short mirrorPos(unsigned short pos);/* Mirror Position: 0 to 1023 */
unsigned short pos2deg(unsigned short pos); /* deg: 0 to 300 */
unsigned short deg2pos(unsigned short deg); /* deg: 0 to 300 */
short  getTailDeg(void);
void setTailDeg(short deg, unsigned short speed);
short  getAbdomenDeg(void);/* deg: -75 to +80 */
short  getAbdomenDegG(void); /* Gets inclination wrt g-plane. deg: -150 to +150 */
void setAbdomenDeg(short deg, unsigned short speed); /* deg: -75 to +80 */
unsigned short dist2cm(unsigned short dist);
float cosML(float x);
float readHeight(void);
void setupAcc(void);/* Configure MCU bits to use Accelerometer */
uint16_t readAcc(unsigned char addr);
float readAccNorm(unsigned char addr);/* read calibrated and normalized (0-1) value from Accel Channel @ addr */
float readAccDeg(unsigned char addr);/* read deg value from Accel on channel @ addr */
void setupSharp(void);/* Configure MCU bits to use Sharp IR Sensors Board */
uint16_t readSharp(unsigned char addr); /* Read value of Sharp sensor mapped on address "addr" */
float readSharp_cm(unsigned char addr); /* Read value of Sharp sensor mapped on address "addr"  and convert to cm distance */
void readAllSensors(Sensors *s); /* Read all sensor values and put into struct s */
void printAllSensors(Sensors sensors);
void USART_Flush(void);

//// Globals ------------------------

//Communication Dynamixel
_Bool comm_err = 0; //Used to ensure correct communication with dynamixel 
//SHARP Sensor
//
// The object of the parameters of GP2Y0A21YK sensor
// 
const ir_distance_sensor GP2Y0A21YK = { 5461, -17, 2 }; //parameters to convert in cm

//Communication
int CommStatus;

//Whegs, the numbers indicate the ID of each wheg
int whegs_sx[3] = {6,3,14};
int whegs_dx[3] = {8,7,12};
		
//Desired Nominal Speed 
unsigned short dnspeed = 0;//280;
	
struct ServoData sdata[7];
unsigned char dCIR; //Center IR Sensor Distance
unsigned char dLIR; //Left IR Sensor Distance
unsigned char dRIR; //Right IR Sensor Distance
unsigned char lCIR; //Center IR Sensor Luminosity
_Bool parallel = 1; //Parallel Mode for Whegs
int parallel_time = 0;//Parallel Config Time var
int frontLift_time = 0;//FrontLift Config Time var
int whegBlocked_time = 0;//Wheg Blocked Time var
_Bool wheg_blocked = 0 ; //Wheg Blocked state
short dpAbd = 0;//Desired Abdomen Inclination wrt Body in [deg]
unsigned short pTail ;//Tail Servo Position 
int dpTail = -1; //-1 : no desidered position (any is good)
_Bool dir15 = 0; //sensor servo (id 15) direction. 0 = CW, 1= CCW.
Sensors sensors;	/* Holds all sensors values */
float x_speed = 0; /* X direction speed , 2 be calc'd from x_acc */

//Button vars
_Bool btn_switch1 = 0;
_Bool btn_switch2 = 0;
	
//Behaviors Controllers
unsigned short turnL = 0;
unsigned short turnR = 0;
_Bool walking = 1;
uint16_t resultX, resultY;
_Bool bRemoteButton = false;

int main(void)
{	
	serial_initialize(57600);
	dxl_initialize( 0, DEFAULT_BAUDNUM ); // Not using device index
	sei();	// Interrupt Enable	
	printf( "\n\n Venom Whegs Sync Master\n\n" );
	_delay_ms(20000);
	// Enable Continuous Turn on whegs servos
	int i;
	for (i=0;i<3;i++){
		// dxl_write_word(<servo number>, <addresses>, <value>)
		dxl_write_word( whegs_sx[i], P_MOVING_SPEED_L, 0 ); // set whegs_sx[i] moving speed to 0 position
		dxl_write_word( whegs_dx[i], P_MOVING_SPEED_L, 1024 ); // set whegs_dx[i] moving speed to 1024 position, inverting the direction of movement
		dxl_write_word( whegs_sx[i], 8, 0 ); // set whegs_sx[i] CCW_ANGLE_LIM to 0 
		dxl_write_word( whegs_dx[i], 8, 0 );
	}	
	
	//Disable Continuous Turn for Body Torsion Servo and Tail Servo
	dxl_write_word( 10, 8, 1023 );//Disable Continuous Turn Mode
	dxl_write_word( 10, P_TORQUE_ENABLE, 0 ); //Disable Torsion Servo
	dxl_write_word( 1, 8, 1023 );//Disable Continuous Turn Mode
	dxl_write_word( 1, P_TORQUE_ENABLE, 0 ); //Disable Tail Servo
	// Write moving speed
	//dxl_write_word( 6, P_MOVING_SPEED_L, 150 );
	//dxl_write_word( 8, P_MOVING_SPEED_L, 1174 );

	//Prepare servos data
	sdata[0].id = whegs_sx[0];
	sdata[1].id = whegs_sx[1];
	sdata[2].id = whegs_sx[2];
	sdata[3].id = whegs_dx[0];
	sdata[4].id = whegs_dx[1];
	sdata[5].id = whegs_dx[2];
	sdata[6].id = 15; //Distance Sensor Servo
	
	dxl_write_word(15, 8, 1023); //Position Mode for Sensor Servo
	
	//Remote Button Init
	DDRE  = 0x0C;
	PORTE = 0xF0;
	//DDRE |= (0<<BTN_RIGHT) ;//MCU PIN BTN_RIGHT set as Input (button has been physically removed)

	//Prepare Sensor
	dxl_write_byte( 100, OBSTACLE_DETECTED_COMPARE, 1 );//High Dist Sens
	//dxl_write_byte(100,OBSTACLE_DETECTED_COMPARE_VALUE, 50) ;//50 is detection threshold 
	
	adc_init(); //This function initializes the ADC hardware and asynchronous ADC read subsystem
	setupAcc();
	setupSharp();
	
	initTimer1();
	walking = false;
	USART_Flush();
	
	while(1)
	{
		// Read Remote Controller via XBee using Virtual Wires
		{
			
			resultX = adc_start ( 4 );
			resultY = adc_start ( 3 );
			bRemoteButton = (PINE & BTN_RIGHT);
			//printf("\r\nresultX resultY button : %u %u %d", resultX, resultY, bRemoteButton);

			// BUTTON
			if (bRemoteButton)
			{
				walking = true;
			}
			else
			{
				walking = false;
			}

			//X
			if ( resultX > 340 )
			{
				turnL = 0; turnR=0; // Go Fwd
			} else
			if ( resultX < 300  )
			{
				turnL = 1; turnR=1; // Go Bwd
			}
			
			//Y
			if ( resultY > 340 )
			{
				turnR = 1; turnL=0; // Turn Right
			}
			else
			if ( resultY < 300 )
			{
				turnL = 1; turnR=0; // Turn Left
			}	
		}
		
		//Read and Communicate Sensors Data 
		readAllSensors(&sensors);
		printAllSensors(sensors);
		
		// Set Speed According to Body Inclination 
		dnspeed = (600 + 4 *sensors.body_inclination_xg); //set speed according to terrain inclination(more speed = more force)
		
		//printf("\r\nturnL turnR walking : %u %u %d %d", turnL, turnR, walking, dnspeed);
		
		//Main Behaviors Section 
		//Vars for Behaviors
		float ld = abs((int)(sensors.distance_floor_left - sensors.distance_floor_center));
		float rd = abs((int)(sensors.distance_floor_right - sensors.distance_floor_center)); 
		
		//Jump to Floor Behavior
		if (sensors.abdomen_inclination_g < -30 && sensors.distance_front_center < 30){ //..isFalling, TerrainIsNear -> JUMP!
			stop();
			comm_err = 0;		//communication error
			while(sensors.abdomen_inclination_g < -30 && sensors.distance_front_center < 30){//JUMP
				//printf("\nJUMP\n");
				if (comm_err == 0 )dpAbd = dpAbd + 1;				
				//LIFT BODY			
				comm_err = 0;//reset communication error
				//setTailDeg(90, 1023); //Lift tail at super speed (back body goes down, front body goes up)
				setAbdomenDeg(dpAbd, 1023);//Lift 
				if (checkDxlConn()==-1)
					comm_err = 1;
				else{
					int isMoving = 1;
					while(isMoving == 1) {//reach pos				
						isMoving = dxl_read_byte(16,P_MOVING);
						if (checkDxlConn()==-1){
							comm_err = 1;
							isMoving = 0; //permits exiting from while
						}
					}
				}
				sensors.abdomen_inclination_g = getAbdomenDegG();
				sensors.distance_front_center = readSharp_cm(4);
			}
			//setTailDeg(0, 512); //put tail in horizontal pos
			//while(dxl_read_byte(1,P_MOVING));//wait pos reached
			//dxl_write_word( 1, P_TORQUE_ENABLE, 0 ); //Disable Tail Servo
		}else
		//Terrain Check Behavior (DISABLE BECAUSE VISION SYSTEM IS INSUFFICIENT)
		if( 0 && (ld > 8 || rd > 8)) {	//terrain not uniform //last=4
			//printf("ld:%d,rd:%d\n",(int)ld,(int)rd);
			if (ld >= rd) {turnR = 1; turnL=0;}
			if (ld < rd) {turnL = 1; turnR=0;}
		}else
		if (sensors.distance_front_center <  15){//Near Center Obstacle..
			stop();//STOP WALKING
			//Approaching Obstacle Behavior
			comm_err = 0;		//communication error
			while(sensors.distance_front_center < 15 && sensors.abdomen_inclination < 80){//LIFT UP..
				//printf("\nLIFT UP TO CLIMB\n");
				if (comm_err == 0 )dpAbd = dpAbd + 1;				
				if (dpAbd > 80)dpAbd = 80;
				//LIFT BODY			
				comm_err = 0;//reset communication error
				setAbdomenDeg(dpAbd, 200);//Lift 
				if (checkDxlConn()==-1)
					comm_err = 1;
				else{
					int isMoving = 1;
					while(isMoving == 1) {//reach pos				
						isMoving = dxl_read_byte(16,P_MOVING);
						if (checkDxlConn()==-1){
							comm_err = 1;
							isMoving = 0; //permits exiting from while
						}
					}
				}
				sensors.distance_front_center = readSharp_cm(4);
			}				
		}
		else //Adapt to Floor Behavior
		if(sensors.distance_floor_center > 23 ){ // Floor too distant .. 
			stop();
			comm_err = 0;		//communication error
			while(sensors.distance_floor_center > 23 && sensors.abdomen_inclination > -75){// LOWER DOWN..
				//printf("\nLOWERING TO THE FLOOR\n");
				if (comm_err == 0 )dpAbd = dpAbd - 1;				
				if (dpAbd < -75)dpAbd = -75;
				comm_err = 0;//reset communication error
				setAbdomenDeg(dpAbd, 200);//Lower Abdomen	
				if (checkDxlConn()==-1)
					comm_err = 1;
				else{
					int isMoving = 1;
					while(isMoving == 1) {//reach pos				
						isMoving = dxl_read_byte(16,P_MOVING);
						if (checkDxlConn()==-1){
							comm_err = 1;
							isMoving = 0; //permits exiting from while
						}
					}
				}
				sensors.distance_floor_center = readSharp_cm(6);
			}
		}
		//End of Behaviors
		//Walking Actions
		if (walking==1){
			if(turnL && turnR){
				int i ;
				for (i=0;i<3;i++){//Go Backward
					dxl_write_word( whegs_sx[i], P_MOVING_SPEED_L, 1624 );
					dxl_write_word( whegs_dx[i], P_MOVING_SPEED_L, 600 );	
					
					//--START (DISABLE)				
					//_delay_ms(40000);//Mantain behavior
					//Disable behavior..
					//turnL = 0;
					//turnR = 0;
					//--END
					
				}		
				
			}else
			if(turnL){//Turn Left
				int i ;
				for (i=0;i<3;i++){
					dxl_write_word( whegs_sx[i], P_MOVING_SPEED_L, 1624 );
					dxl_write_word( whegs_dx[i], P_MOVING_SPEED_L, 1624 );
					//dxl_write_word( whegs_dx[i], P_MOVING_SPEED_L, 0 );
					
					//--START (DISABLE)	
					//_delay_ms(20000);//Mantain behavior
					//Disable behavior..
					//turnL = 0;
					//turnR = 0;
					//--END
				}
			}else
			if(turnR){//Turn Right 
				int i ;
				for (i=0;i<3;i++){
					dxl_write_word( whegs_sx[i], P_MOVING_SPEED_L, 600 );
					//dxl_write_word( whegs_sx[i], P_MOVING_SPEED_L, 0 );
					dxl_write_word( whegs_dx[i], P_MOVING_SPEED_L, 600 );
					
					//--START (DISABLE)	
					//_delay_ms(20000);//Mantain behavior
					//Disable behavior..
					//turnL = 0;
					//turnR = 0;
					//--END
				}		
			}else
			if (!(turnL + turnR)){//If not turning				
				go_fwd();//Restart walking			
			}
		}else{//Stop Walking
			stop();
		}							
		//Tail Control Section
		pTail = (unsigned short)dxl_read_word( 1, P_PRESENT_POSITION_L);
		//printf("%d ",pTail);
		
		//Tail Behaviors Manager
		// Check if desidered Tail position (if was set) has been reached
		if(dpTail != -1 && abs((int)dpTail - (int)pTail) < 50){
			dxl_write_word( 1, P_TORQUE_ENABLE, 0 ); //Disable Tail Servo
												// Needed to shut off tail behaviors
												//activated in previous loop cycle
			dpTail = -1; //disable desidered Tail position (any will be good)
		}
		
		//Tail Behavior 1: Avoiding Falling Backward
		//If very high tail and abdomen not low and body tilted up
		if (parallel && pTail < 300 && getAbdomenDeg() >-10 && readAccDeg(1) > 45){//avoid falling backward
			//printf("\navoid falling backward\n");
			dpTail = 500;
			dxl_write_word( 1, P_GOAL_POSITION_L, dpTail );
			dxl_write_word( 1, P_MOVING_SPEED_L, 210 );
		}
		
		//Tail Behavior 2: Climbing 
		//If quite high tail and abdomen quite low and body tilted up
		if (parallel && pTail < 450 && getAbdomenDeg() < -20 && readAccDeg(1) > 60 ){//climb
			//printf("\nclimb\n");
			dpTail = 712;
			dxl_write_word( 1, P_GOAL_POSITION_L, dpTail );
			dxl_write_word( 1, P_MOVING_SPEED_L, 210 );
		}else
	
		
		//Working on timer..
// 		time = Timer1Read();
// 		//printf(" %u ", time/(unsigned int)TIME_MILLISEC);
// 		printf(" %u ", time);
		//printf(" main ");
		//printf("\n");
		
		if(0){//STOP MOTION
			//printf("PRESS KEY\n");
			torque(0);
			unsigned char ReceivedData = getchar();
			torque(1);
		}
		
	}	//main loop	

	return 0;
}


void printAllSensors(Sensors sensors){
	//printf("dnspeed:\t%d\t",dnspeed);
	printf("FC %d\t",(int)(sensors.distance_front_center));
	printf("GC %d\t",(int)(sensors.distance_floor_center));
	printf("GL %d\t",(int)(sensors.distance_floor_left));
	printf("GR %d\t",(int)(sensors.distance_floor_right));
	printf("Z %d\t",(short)(1000*readAccNorm(0)));
	printf("X %d\t",(short)(1000*readAccNorm(1)));
	printf("Y %d\t",(short)(1000*readAccNorm(2)));
	printf("AD %d\t",getAbdomenDeg());
	printf("TD %d\n",getTailDeg());
}



// Print communication result
void PrintCommStatus(int CommStatus)
{
	switch(CommStatus)
	{
	case COMM_TXFAIL:
		printf("COMM_TXFAIL: Failed transmit instruction packet!\n");
		break;

	case COMM_TXERROR:
		printf("COMM_TXERROR: Incorrect instruction packet!\n");
		break;

	case COMM_RXFAIL:
		printf("COMM_RXFAIL: Failed get status packet from device!\n");
		break;

	case COMM_RXWAITING:
		printf("COMM_RXWAITING: Now recieving status packet!\n");
		break;

	case COMM_RXTIMEOUT:
		printf("COMM_RXTIMEOUT: There is no status packet!\n");
		break;

	case COMM_RXCORRUPT:
		printf("COMM_RXCORRUPT: Incorrect status packet!\n");
		break;

	default:
		printf("This is unknown error code!\n");
		break;
	}
}

// Print error bit of status packet
void PrintErrorCode()
{
	if(dxl_get_rxpacket_error(ERRBIT_VOLTAGE) == 1)
		printf("Input voltage error!\n");

	if(dxl_get_rxpacket_error(ERRBIT_ANGLE) == 1)
		printf("Angle limit error!\n");

	if(dxl_get_rxpacket_error(ERRBIT_OVERHEAT) == 1)
		printf("Overheat error!\n");

	if(dxl_get_rxpacket_error(ERRBIT_RANGE) == 1)
		printf("Out of range error!\n");

	if(dxl_get_rxpacket_error(ERRBIT_CHECKSUM) == 1)
		printf("Checksum error!\n");

	if(dxl_get_rxpacket_error(ERRBIT_OVERLOAD) == 1)
		printf("Overload error!\n");

	if(dxl_get_rxpacket_error(ERRBIT_INSTRUCTION) == 1)
		printf("Instruction code error!\n");
}

unsigned int Timer1Read( void )
{
unsigned char sreg;
unsigned int i;
/* Save global interrupt flag */
sreg = SREG;
/* Disable interrupts */
cli() ;//__disable_interrupt();
/* Read TCNTn into i */
i = TCNT1;
/* Restore global interrupt flag */
SREG = sreg;
return i;
}


void Timer1Write( unsigned int i )
{
unsigned char sreg;
/* Save global interrupt flag */
sreg = SREG;
/* Disable interrupts */
cli();//__disable_interrupt();
/* Set TCNTn to i */
TCNT1 = i;
/* Restore global interrupt flag */
SREG = sreg;
}

void Timer1Pause(unsigned int flag){/*Pause using timer 1 */
	Timer1Write(0);
	unsigned int time = Timer1Read();
	while(time < flag ){
		time = Timer1Read();
	}
}

void initTimer1(void){
	TCCR1B = (1<<CS12)|(1<<CS10);//Prescaler to Sys Clock / 1024 = (16MHz / 1024)
}

void init_OC1A(void)
{
//ASSR= 1<<AS2; // Enable asynchronous mode
TCCR1B = (1<<CS12)|(1<<CS10);//Prescaler to Sys Clock / 1024 = (16MHz / 1024)
TIFR1 = 1<<OCF1A; // Clear OCF1/ Clear pending interrupts
TIMSK1= 1<<OCIE1A; // 1:Enable (0:Disable) Timer1 Output Compare Mode Compare Match Interrupt
OCR1A= 48; // Set Output Compare Value to 32
//Timer1Write((unsigned int) 0);//Resets Timer
//DDRB= 0xFF; // Set Port B as output
//while (ASSR&(1<<OCR2UB)); // Wait for registers to update
}

//Interrupt Function


ISR(TIMER1_COMPA_vect){
	// Clear timer on compare match (now manually)
	Timer1Write((unsigned int) 0);//Resets Timer
	float deltaT = 1000/dnspeed; //msecs per count (AX12 has 1023 counts for 300 deg)
	OCR1A = TIME_MILLISEC * deltaT;// set compare interval
}


unsigned short mirror(unsigned short speed){//calc specular speed
	if (speed > 1023)
		return speed - 1024;
	else
		return speed + 1024;
}

void adc_init(void){ // Function used to initialise the ADC feature
	ADMUX=0X40; // 0x40 for 10bit //0x60 for 8bit
	ADCSRA=0X87; // We have set the ADSC bit to start a conversion, and the
	// ADPS bits are set so that the prescaler is 128
	//ADCSRA=0X80; // ADEN is set, to enable the ADC
}

uint16_t adc_start(unsigned char channel){ // Function to perform an ADC conversion, Takes 0-8 as input
	// to select which input to convert
	unsigned char i;
	i=channel&0x07; // Decide which line to perform ADC conversion on
	//ADMUX=i|0x60; // Enter which line to perform in the ADC control register (for 8 bit)
	ADMUX=i|0x40; // Enter which line to perform in the ADC control register (for 10 bit)
	ADCSRA|=1<<ADSC;
   
	while(ADCSRA & (1<<ADSC));  // wait for conv. to complete
	//unsigned char temp=ADCH; //for 8-bit
	unsigned short temp;
	temp=ADC;//for 10-bit
	return temp;
}

int checkDxlConn(void){ //checkDxlConn get the result and if it has success it prints the status, if not it returns -1
	CommStatus = dxl_get_result();
	if( CommStatus != COMM_RXSUCCESS ){
 		PrintCommStatus (CommStatus);
 		PrintErrorCode();
		 return -1;
	}
	return 1;	
}

unsigned short mirrorPos(unsigned short pos){/* Calc pos for upside-down servo */
	pos = 1023 - pos;
	return pos;
}

unsigned short pos2deg(unsigned short pos){ /* deg : 0 to 300 . 150 is center */
	if(pos > 1023)return -1; /* Error: out of range */
	//printf(" p%u ",pos);
	uint32_t deg;
	//unsigned int deg;
	deg = (uint32_t)pos;
	//printf(" d%u ",deg);
	deg = deg * 293; //it was 29.3
	//printf(" d%u ",deg);
	deg = deg / 1000;
	//printf(" d%u ",deg);
	return (unsigned short)deg;
	
}
unsigned short deg2pos(unsigned short deg){/* deg : 0 to 300 . 150 is center */
	if (deg > 300) return -1 ; /* Error: out of range */
	//printf(" p%u ",deg);
	uint32_t pos;
	//unsigned int deg;
	pos = (uint32_t)deg;
	//printf(" d%u ",pos);
	pos = pos * 1000; 
	//printf(" d%u ",pos);
	pos = pos / 293;//it was 29.3
	//printf(" d%u ",pos);
	return (unsigned short)pos;
	
}

short  getTailDeg(void){
	short deg = (short)pos2deg(dxl_read_word(1, P_PRESENT_POSITION_L)) - 150;
	return  -deg; //servo is mounted upside-down
}
void setTailDeg(short deg, unsigned short speed){
	deg = - deg; //servo is mounted upside-down
	if (deg < -90 || deg > 90 ){/* Error: out of range */
		printf("\nError: deg must be in the -90 to 90 deg range.\n");
	} else{
		unsigned short posTail = -1;//holds abdomen joint position
		posTail = deg2pos((unsigned short)(deg + 150));
		//printf ("1pos:\t%d\t",posTail);
		dxl_write_word(1, P_MOVING_SPEED_L, speed);
		dxl_write_word(1, P_GOAL_POSITION_L, posTail);
		posTail = mirrorPos(posTail ) ;
		//printf ("5pos:\t%d\n",posTail);
		dxl_write_word(5, P_MOVING_SPEED_L, speed);
		dxl_write_word(5, P_GOAL_POSITION_L, posTail);
	}
}

short  getAbdomenDeg(void){
	return (short)pos2deg(dxl_read_word(16, P_PRESENT_POSITION_L)) - 150;
}

short  getAbdomenDegG(void){ /* Gets inclination wrt g-plane */
	return getAbdomenDeg() + (short)readAccDeg(1);
}


void setAbdomenDeg(short deg, unsigned short speed){
	if (deg < -75 || deg > 80 ){/* Error: out of range */
		printf("\nError: deg must be in the -75 to + 80 deg range.\n");
	} else{
		unsigned short posAbdomen = -1;//holds abdomen joint position
		posAbdomen = deg2pos((unsigned short)(deg + 150));
		//printf ("16pos:\t%d\t",posAbdomen);
		dxl_write_word(16, P_MOVING_SPEED_L, speed);
		dxl_write_word(16, P_GOAL_POSITION_L, posAbdomen);
		posAbdomen = mirrorPos(posAbdomen ) ;
		//printf ("18pos:\t%d\n",posAbdomen);
		dxl_write_word(18, P_MOVING_SPEED_L, speed);
		dxl_write_word(18, P_GOAL_POSITION_L, posAbdomen);
	}
}

unsigned short dist2cm(unsigned short dist){//Convert distance from SHARP GP2Y0A21YK sensor to cm
	uint32_t cm = (uint32_t)dist;
	cm = 12344 * cm ;
	return cm;
}	


 
//
// Converting the values of the IR distance sensor to centimeters
// Returns -1, if the conversion did not succeed
//
float ir_distance_calculate_cm(ir_distance_sensor sensor, unsigned short adc_value){
	if (adc_value + sensor.b <= 0)
	{
		return -1;
	}
	float dist_cm;
	dist_cm = (float)sensor.a / (float)(adc_value + sensor.b) - (float)sensor.k;
	//dist_cm =  5461 / (adc_value + -17) - 2;
	//printf("cm%d ",dist_cm);
	return dist_cm;
}

float cosML(float x){ //cosine MacLaurin 4 term  (GOOD from 0∞ to 45∞ ,need to improve to 50∞)
	//deg 2 rad
	x = x * 0.017453;
	//printf("r%d ",(int)(x*10));
	//cos(x):  1-0.5x^2 +  0.04167 * x^4
	float cosine;
	cosine = 1- 0.5 * x*x + 0.04167 * x *x *x *x ;
	//printf("cos%d ",(int)(cosine*10));
	return cosine;	
}

float readHeight(void){
	//Read Sensor
	uint16_t result=adc_start(6);	// cHANNEL 6
	//printf("r%u ", result);
	float dist_cm;
	dist_cm = ir_distance_calculate_cm(GP2Y0A21YK, result);//Convert
	//printf("cm%d ", (int)(dist_cm*10));
	float height ;
	height = 4.5 +  (float)dist_cm;//Add sensor stand length to Height
	if(height<10) height = 10;
	else if(height>80) height = 80;
	return height ;
}	

void go_fwd(void){
	printf ( "\r\n GO FWD dnspeed=%d", dnspeed);
	int i;
	for (i=0;i<3;i++)
		dxl_write_word( sdata[i].id, P_MOVING_SPEED_L, dnspeed );	
	for (i=3;i<6;i++)
		dxl_write_word( sdata[i].id, P_MOVING_SPEED_L, mirror(dnspeed ));			
}

void stop(void){
	int i ;
	for (i=0;i<3;i++)
			dxl_write_word( sdata[i].id, P_MOVING_SPEED_L, 0 );	
	for (i=3;i<6;i++)
			dxl_write_word( sdata[i].id, P_MOVING_SPEED_L, mirror(0));				
}

void torque(unsigned char val){
	int i ;
	for (i=0;i<3;i++)
		dxl_write_word( sdata[i].id, P_TORQUE_ENABLE, val );	
	for (i=3;i<6;i++)
		dxl_write_word( sdata[i].id, P_TORQUE_ENABLE, val);				
}



unsigned char readAXS1cd(void){//Read AX-S1 Sensor Center Distance Filtering Errors and Compensating Ambient Light
	int i,dCIR,lCIR,cdCIR,dCIRmin;//corrected distance
	dCIRmin = 255;
	for(i=0;i<10;i++){//Collect min val (avoid ambient ir-modulated interferences )
		do{	dCIR = (unsigned char) dxl_read_byte( 100, CENT_IR_SENSOR_DATA);}
			while (dxl_get_result()!=COMM_RXSUCCESS);	
		if (dCIR < dCIRmin) dCIRmin = dCIR;			
	}//using dCIRmin as distance value	
	do{	lCIR = (unsigned char) dxl_read_byte(100,CENTER_LUMINOSITY);}
		while (dxl_get_result()!=COMM_RXSUCCESS);
	cdCIR = dCIRmin - 2*lCIR ;//Compensate Ambient Luminosity
	if(cdCIR <0)cdCIR = 0;//Lower bound
	//printf("dCIRmin %d \t",dCIRmin);
	//printf("lCIR %d \t",lCIR); 
	//printf("cdCIR %d \n",cdCIR); 
	//if(dCIRmin > 100)printf(" >100 \n");
	return cdCIR;
}


void setupAcc(void){/* Configure MCU bits to use Accelerometer */
	//Configure MCU outputs to interface Acc inputs
	DDRA |= (1<<PA7)|(1<<PA6) ;//MCU PIN A6 and A7 set as Output 
}

uint16_t readAcc(unsigned char addr){
	/* Note: All PORTA pin values are inverted (1 is LOW, 0 is HIGH) 
	 * This is due to cm-510 output transistor, I guess
	 */
	switch (addr){
	case 0 : 
		// Choosing Z Accel.
		PORTA = (1<<PA7)|(1<<PA6); //MPLEX: ADDR 00 (0)
		//printf("\n00:Z \t");
		break;
	
	case 1 : 
		// Choosing X Accel.
		PORTA = (1<<PA7)|(0<<PA6); //MPLEX: ADDR 01 (1)
		//printf("\n01:X \t");
		break;
	
	case 2 :
		// Choosing Y Accel.
		PORTA = (0<<PA7)|(1<<PA6); //MPLEX: ADDR 10 (2)
		//printf("\n10:Y\t");
		break;
	
	case 3 :
		// Chosing Free Address
		PORTA = (0<<PA7)|(0<<PA6); //MPLEX: ADDR 11 (3)
		//printf("\n11:F\t");
		break;
	default:
			printf("\nWrong Address (%d) Selected on channel 2\n", addr);
			return -1;
			
	}	
	
	//Pause before acquiring..
	Timer1Write(0);
	int time = Timer1Read();
	while(time < TIME_MILLISEC){
		time = Timer1Read();
		//printf("time=%d\n",time);
	}
	int16_t result=adc_start(2);	//Channel 2
	//printf("result : \t%d\n",result);
	return result;
}

void setupSharp(void){/* Configure MCU bits to use Sharp IR Sensors Board */
	//Configure MCU outputs to interface Board inputs
	DDRA |= (1<<PA2)|(1<<PA3);//MCU PIN PA2 and PA3 set as Output 
	DDRF |= (1<<PF5) ;//MCU PIN PF4 set as Output 
}

uint16_t readSharp(unsigned char addr){ /* Read value of Sharp sensor mapped on address "addr" */
	/* Note: All PORTA pin values are inverted (1 is LOW, 0 is HIGH) 
	 * This is due to cm-510 output transistor, I guess
	 */
	switch(addr){
		case 0:	//MPLEX: ADDR 000 (0)
			PORTA = (1<<PA3)|(1<<PA2); //BIT1:0 = 00
			PORTF = (0<<PF5);//BIT2 = 0
			break;
		case 1:	//MPLEX: ADDR 001 (1)
			PORTA = (1<<PA3)|(0<<PA2); //BIT1:0 = 01
			PORTF = (0<<PF5);//BIT2 = 0
			break;
		case 2:	//MPLEX: ADDR 010 (2)
			PORTA = (0<<PA3)|(1<<PA2); //BIT1:0 = 10
			PORTF = (0<<PF5);//BIT2 = 0
			break;
		case 3:	//MPLEX: ADDR 011 (3)
			PORTA = (0<<PA3)|(0<<PA2); //BIT1:0 = 11
			PORTF = (0<<PF5);//BIT2 = 0
			break;
		case 4:	//MPLEX: ADDR 100 (4)
			PORTA = (1<<PA3)|(1<<PA2); //BIT1:0 = 00
			PORTF = (1<<PF5);//BIT2 = 1
			break;
		case 5:	//MPLEX: ADDR 101 (5)
			PORTA = (1<<PA3)|(0<<PA2); //BIT1:0 = 01
			PORTF = (1<<PF5);//BIT2 = 1
			break;
		case 6:	//MPLEX: ADDR 110 (6)
			PORTA = (0<<PA3)|(1<<PA2); //BIT1:0 = 10
			PORTF = (1<<PF5);//BIT2 = 1
			break;
		case 7:	//MPLEX: ADDR 111 (7)
			PORTA = (0<<PA3)|(0<<PA2); //BIT1:0 = 11
			PORTF = (1<<PF5);//BIT2 = 1
			break;
		default :
			printf("\nWrong Address (%d) Selected on channel 2\n", addr);
			return -1;
	}
	//Pause before acquiring..
	Timer1Write(0);
	int time = Timer1Read();
	while(time < TIME_MILLISEC ){
		time = Timer1Read();
		//printf("time=%d\n",time);
	}
	int16_t result = adc_start(6);	//Channel 6
	//filtering : pass only 0.3V (64) - 3V (613)
	if (result < 64)
		result = 64;
	if (result > 613)
		result = 613;
	//printf("result : \t%d\n",result);
	return result;
}

unsigned short sharp_calib[17][2]={
{63,40},
{71,35},
{85,30},
{106,25},
{132,20},
{153,18},
{165,16},
{192,14},
{214,12},
{257,10},
{286,9},
{319,8},
{360,7},
{415,6},
{480,5},
{562,4},
{613,3}
}; 

float readSharp_cm(unsigned char addr){ /* Read value of Sharp sensor mapped on address "addr"  and convert to cm distance */
	float val_cm;
	unsigned short val_10bit = readSharp(addr);
	int i;
	for(i = 0; i<17; i++){
		if (val_10bit >=  sharp_calib[i][0] && val_10bit < sharp_calib[i+1][0]){
			val_cm = (float)sharp_calib[i][1] + (float)((float)val_10bit - (float) sharp_calib[i][0])* (float)((float)((float)sharp_calib[i+1][1]- (float)sharp_calib[i][1])/(float)((float)sharp_calib[i+1][0] - (float)sharp_calib[i][0]));
			return val_cm;
		}
	}
	return -1; /* Error, data is out of range */
}

float readAccNorm(unsigned char addr){/* read calibrated and normalized (0-1) value from Accel Channel @ addr */
	float norm_val = 0; 
	if (addr == 1){ /* Calibr for X */
		norm_val = (float)((float)readAcc(addr) - (float)325)/(float)164; //325 : 0-val , 164 : mod(g)
	}else
	if (addr == 2){ /* Calibr for Y */
		norm_val =  (float)((float)readAcc(addr) - (float)338)/(float)162; //339 : 0-val , 164 : mod(g)
	}else
	if (addr == 0){ /* Calibr for Z */
		norm_val =  (float)((float)readAcc(addr) - (float)286)/(float)164; //286 : 0-val , 164 : mod(g)
	}
	else{/* Wrong Channel (addr) selected */
		return -1;
	}
	/* If you are here then the channel was valid .. */	
	//Perform value bounding..
	if (norm_val > 1) norm_val = 1;
	else if (norm_val < -1) norm_val = -1;
	return norm_val;
		
}

float readAccDeg(unsigned char addr){/* read deg value from Accel on channel @ addr */
	float val = readAccNorm(addr);
	//printf("\n\tnormval=%d\n", (int)(1000*val));
	val = 	asin(val);
	//printf("\n\tasinval=%d\n", (int)(1000*val));	
	val = val * 57.296739;//rad2deg
	return val;
}
	

void readAllSensors(Sensors *s){
	s->distance_front_center = readSharp_cm(4);
	s->distance_floor_center = readSharp_cm(6);
	s->distance_floor_left = readSharp_cm(5);
	s->distance_floor_right = readSharp_cm(7);
	s->body_inclination_xg = readAccDeg(1);
	s->body_inclination_yg = readAccDeg(2);
	s->body_inclination_zg = readAccDeg(0);
	s->abdomen_inclination = getAbdomenDeg();
	s->abdomen_inclination_g = getAbdomenDegG();	
}
	
	


int facingAbyss(void){
	//TODO 
	return -1;
}


//SERIAL
void USART_Flush( void )
{
	unsigned char dummy;
	while ( UCSR1A & (1<<RXC1) ) dummy = UDR1;
}